(* ===================================================================

   A functor to build the actual syntax checker out of its 
   specification.

=================================================================== *)


local 

open Parse HolKernel boolLib 
open Option Err 
open L0Theory L0_emb_mldata L0_emb_basic 
open SChecker 

in

functor build_SC (SC : SChecker) =

   struct

   local

   fun println s = (print "\n"; print s)

   val SC_instr = fold_INSTR SC.scAlgebra

   in

   fun print_msgs () 
       =
       ( println ""
       ; (let
          val msgs = List.rev (SC.get_SCmsgs ())
          in
          if null msgs then (println "No syntax errors found" ; println "")
          else ( println "The following syntax errors/warnings are found:"
              ; println ""
              ; List.map SC.print_SCmsg msgs
              ; () )
          end)
       ; println ""
       )

   fun check progdef_thm
        =
        ( SC.initializeState ()
        ; ( let
            val prg = (buildPROG
                       o concl
                       o SPEC_ALL) 
                         progdef_thm
            in
            ( (  SC.add_SCmsgs
               o List.concat
               o List.map SC.check_allowedtype ) 

                (#prparams prg  @ #pvparams prg )

            ; SC.add_SCmsgs (SC.check_assertion (#ppre prg))
            ; (SC_instr o buildINSTRtree o #pbody) prg
            ; SC.add_SCmsgs (SC.check_assertion_ret (#ppost prg)))
            end
            handle 
            L0Exn e => SC.add_SCmsgs [SC.L0Error2SCmsg e]
          )
        ; print_msgs ()  )

   end (* of inner local *)
   end (* of struct *)

end