(* ===================================================================

   The top level test-engine, represented as a functor

=================================================================== *)


local 

(* open Parse HolKernel boolLib *)

open PP Err 
     Test_emb_mldata Test_emb_basic Test_Engine

in

functor build_TE (TEspec : Test_Engine) =

   struct

   local

   fun tengine test pps =

        let

        val tsuite = buildTEST (remove_whitebox test)

        fun pp_tsuite pps =  fold_TEST (TEspec.teAlg) tsuite "root" pps
                           (* the "root" initialize a parameter which can
                              be thought as an inherited attr intended
                              to pass the path from root to a particular
                              node *)

        in

       ( clear_ppstream pps
       ; TEspec.ppte pp_tsuite pps
       ; flush_ppstream pps )

       end
  
       handle L0Exn e => (printL0Error e; raise (L0Exn e))

   in

   (* producing the translation as a string *)

   fun toString test = 
       let
       fun p pps t = tengine t pps
       in pp_to_string 100 p test end
       

   (* just printing the translation to the standard output *)

   fun print test = BasicIO.print (toString test)

   (* translate and save it to a file *)

   fun generate test fname 
       =
       let 
       val f = TextIO.openOut fname
       in
       ( TextIO.output(f, toString test) 
       ; TextIO.closeOut f )
       end 

   end (* of inner local *)
   end (* of struct *)

end